package core

import (
	"git.akyoto.dev/cli/q/src/asm"
	"git.akyoto.dev/cli/q/src/expression"
	"git.akyoto.dev/cli/q/src/types"
)

// CompileDelete compiles a `delete` function call which deallocates a struct.
func (f *Function) CompileDelete(root *expression.Expression) error {
	parameters := root.Children[1:]
	variableName := parameters[0].Token.Text(f.File.Bytes)
	variable := f.VariableByName(variableName)
	defer f.UseVariable(variable)
	f.SaveRegister(f.CPU.Input[0])
	f.SaveRegister(f.CPU.Input[1])
	f.RegisterRegister(asm.MOVE, f.CPU.Input[0], variable.Register)
	f.RegisterNumber(asm.MOVE, f.CPU.Input[1], int(variable.Type.(*types.Pointer).To.TotalSize()))

	for _, register := range f.CPU.General {
		if f.RegisterIsUsed(register) {
			f.Register(asm.PUSH, register)
		}
	}

	f.Call("mem.free")

	for i := len(f.CPU.General) - 1; i >= 0; i-- {
		register := f.CPU.General[i]

		if f.RegisterIsUsed(register) {
			f.Register(asm.POP, register)
		}
	}

	return nil
}
