package core

import (
	"git.akyoto.dev/cli/q/src/ast"
	"git.akyoto.dev/cli/q/src/errors"
	"git.akyoto.dev/cli/q/src/types"
)

// CompileReturn compiles a return instruction.
func (f *Function) CompileReturn(node *ast.Return) error {
	defer f.Return()

	if len(node.Values) == 0 {
		return nil
	}

	if len(node.Values) != len(f.ReturnTypes) {
		return errors.New(&errors.ReturnCountMismatch{Count: len(node.Values), ExpectedCount: len(f.ReturnTypes)}, f.File, node.Values[0].Token.Position)
	}

	for i := len(node.Values) - 1; i >= 0; i-- {
		typ, err := f.ExpressionToRegister(node.Values[i], f.CPU.Output[i])

		if err != nil {
			return err
		}

		if !types.Check(typ, f.ReturnTypes[i]) {
			return errors.New(&errors.TypeMismatch{
				Encountered:   typ.UniqueName(),
				Expected:      f.ReturnTypes[i].UniqueName(),
				ParameterName: "",
				IsReturn:      true,
			}, f.File, node.Values[i].Token.Position)
		}
	}

	return nil
}
